use piston_window::*;
use std::time::Instant;
use rand::Rng;
use std::cmp::max;

pub struct Target {
    position_x: f64,
    position_y: f64,
    size: f64,
    speed_x: f64,
    speed_y: f64,
    last_update: Instant,
    color: types::Color,
    points: u64
}

impl Target {

    pub fn new(position_x: f64, position_y: f64, size: f64, speed_x: f64, speed_y: f64,
               last_update: Instant, color: types::Color, points: u64) -> Self {
        Self {
            position_x,
            position_y,
            size,
            speed_x,
            speed_y,
            last_update,
            color,
            points
        }
    }

    pub fn generate_random(window_width: u64) -> Self {
        let mut rng = rand::thread_rng();
        let target_type = rng.gen_range(0..3);

        let position_x;
        //setting starting y pos
        let position_y;
        let size;
        let speed_x= 0.0;
        let speed_y;
        let color; //red
        let points;

        if target_type == 0 { //red - fast, small
            speed_y = 100.0;
            size = 25.0;
            color = [1.0, 0.0, 0.0, 1.0];
            let max_range_x = max(0, window_width-(size as u64));
            position_x = rng.gen_range(0..max_range_x) as f64;
            points = 100;

        }
        else if target_type == 1 { //green - medium, normal speed
            speed_y = 50.0;
            size = 50.0;
            color = [0.0, 1.0, 0.0, 1.0];
            let max_range_x = max(0, window_width-(size as u64));
            position_x = rng.gen_range(0..max_range_x) as f64;
            points = 50;
        }
        else {
            speed_y = 25.0;
            size = 100.0;
            color = [0.0, 0.0, 1.0,1.0];
            let max_range_x = max(0, window_width-(size as u64));
            position_x = rng.gen_range(0..max_range_x) as f64;
            points = 25;
        }

        position_y = -size;
        return Target::new(
            position_x,
            position_y,
            size,
            speed_x,
            speed_y,
            Instant::now(),
            color,
            points
        );
    }

    pub fn get_points(&self) -> u64 {
        self.points
    }

    pub fn set_position(&mut self, new_x: &f64, new_y: &f64) -> () {
        self.position_x = *new_x;
        self.position_y = *new_y;
    }

    pub fn get_position(&self) -> (f64, f64) {
        (self.position_x, self.position_y)
    }

    pub fn draw(&self,  window: &mut PistonWindow, e: &piston_window::Event) -> () {
        window.draw_2d(e, |context, graphics, _device| {
            rectangle(self.color,
                      [self.position_x, self.position_y, self.size, self.size],
                      context.transform,
                      graphics);
        });
    }

    pub fn update(&mut self) -> () {
        let time_diff = self.last_update.elapsed().as_millis() as f64;
        let (act_x, act_y) = Target::get_position(self); // c compiler wont scream fo unused function get_position (didnt want to add #[allow(dead_code)])
        let new_x = act_x + (self.speed_x * time_diff)/ 1000.0;
        let new_y = act_y + (self.speed_y * time_diff)/ 1000.0;//bcs time dif is in ms and speed is pixels/sec
        Target::set_position(self, &new_x, &new_y);
        self.last_update = Instant::now()
    }

    pub fn is_hit(&self, mouse_x: &f64, mouse_y: &f64) -> bool {
        if *mouse_x >= self.position_x && *mouse_x <= self.position_x + self.size {
            if *mouse_y >= self.position_y && *mouse_y <= self.position_y + self.size {
                return true
            }
        }
        return false
    }

     pub fn is_inside_window(&self, window_width: &f64, window_height: &f64) -> bool {
        if self.position_y < -self.size || self.position_y > *window_height ||  self.position_x < -self.size || self.position_x > *window_width{
            return false
        }
        return true
    }

}
